import yaml
from noetl.payload import Payload, AppConst


class Playbook:
    METADATA_EVENT_TYPE = AppConst.METADATA_EVENT_TYPE
    METADATA_COMMAND_TYPE = AppConst.METADATA_COMMAND_TYPE

    def __init__(self, payload: Payload):
        self.payload = payload
        self.template = None
        self.transitions = None

    async def load_template(self):
        playbook_path = self.payload.get_value("playbook.path")
        playbook_bucket = self.payload.get_value("playbook.bucket")
        kv_playbook = await self.payload.kv_get_decoded(playbook_bucket, playbook_path)
        if kv_playbook:
            self.template = kv_playbook

    def set_transitions(self):
        self.transitions = self.template.get_value("spec.transitions", {})

    def validate_transition(self, current_state, next_state):
        states = self.transitions.get(current_state)

        if states is None:
            raise ValueError(f"No transitions defined for the state: {current_state}")

        elif isinstance(states, list):
            if next_state not in states:
                raise ValueError(f"Transition to '{next_state}' from '{current_state}' is not allowed.")
        elif next_state != states:
            raise ValueError(f"Transition to '{next_state}' from '{current_state}' is not allowed.")

        return True

    def transition(self, current_state, next_state):
        if self.validate_transition(current_state, next_state):
            self.payload.set_status(state=next_state)



import unittest
from unittest.mock import Mock, MagicMock
from playbook import Playbook


class TestPlaybook(unittest.TestCase):
    def setUp(self):
        self.mock_payload = Mock()

        self.MOCK_DATA = {
            'spec': {
                'transitions': {
                    'INITIALIZED': ['RUNNING', 'FAILED', 'TERMINATED'],
                    'RUNNING': ['SUSPENDED', 'COMPLETED', 'TERMINATED', 'FAILED'],
                    'SUSPENDED': ['RUNNING', 'TERMINATED', 'FAILED'],
                    'COMPLETED': ['INITIALIZED'],
                    'FAILED': ['INITIALIZED'],
                    'TERMINATED': ['INITIALIZED']
                }
            }
        }

        self.playbook = Playbook(self.mock_payload)
        self.playbook.template = MagicMock()
        self.playbook.template.get_value.return_value = self.MOCK_DATA['spec']['transitions']

    def test_initialize(self):
        self.assertEqual(self.playbook.payload, self.mock_payload)

    def test_set_transitions(self):
        self.playbook.set_transitions()
        self.assertEqual(self.playbook.transitions, self.MOCK_DATA['spec']['transitions'])

    def test_validate_transition(self):
        self.playbook.set_transitions()

        self.assertTrue(self.playbook.validate_transition('RUNNING', 'COMPLETED'))
        self.assertTrue(self.playbook.validate_transition('SUSPENDED', 'RUNNING'))

        with self.assertRaises(ValueError):
            self.playbook.validate_transition('COMPLETED', 'RUNNING')
            self.playbook.validate_transition('SUSPENDED', 'COMPLETED')

    def test_transition(self):
        self.playbook.set_transitions()

        self.playbook.transition('RUNNING', 'SUSPENDED')
        self.playbook.payload.set_status.assert_called_once_with(state='SUSPENDED')

        with self.assertRaises(ValueError):
            self.playbook.transition('RUNNING', 'INITIALIZED')


if __name__ == "__main__":
    unittest.main()



import unittest
from noetl.payload import Payload, ExecutionState


class TestPayload(unittest.TestCase):
    def setUp(self):
        self.payload = Payload()

    def test_update_status_with_dict(self):
        self.payload.set_status(status={'test_status': 'ACTIVE'})
        self.assertEqual(self.payload.get_value('status.test_status'), 'ACTIVE')

    def test_update_status_with_state(self):
        self.payload.set_status(state='COMPLETED')
        self.assertEqual(self.payload.get_value('status.state'), ExecutionState['COMPLETED'].value)

    def test_update_status_with_invalid_state(self):
        with self.assertRaises(ValueError):
            self.payload.set_status(state='INVALID_STATE')

    def test_update_status_with_dict_invalid_sate(self):
        self.payload.set_status(status={'test_status': 'ACTIVE'}, state='COMPLETED')
        self.assertEqual(self.payload.get_value('status.test_status'), 'ACTIVE')
        self.assertEqual(self.payload.get_value('status.state'), ExecutionState['COMPLETED'].value)

if __name__ == "__main__":
    unittest.main()
